/******************************************************************************

  The DUNE (see www.dune-project.org) module dune-spgrid provides a structured,
  parallel grid satisfying the dune-grid interface.

  Copyright (C) 2012 - 2014 Christoph Gersbacher
  Copyright (C) 2010 - 2013 Robert Klöfkorn
  Copyright (C) 2013        Tobias Malkmus
  Copyright (C) 2009 - 2014 Martin Nolte

  The dune-spgrid module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation, either version 2 of 
  the License, or (at your option) any later version.

  The dune-spgrid module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

#ifndef DUNE_SPGRID_MISC_HH
#define DUNE_SPGRID_MISC_HH

/** \file
 *  \author Martin Nolte
 *  \brief  miscellaneous helper functions
 */

namespace Dune
{

  /** \brief count the number of set bits within an integer
   *
   *  \param[in]  i  integer whose set bits shall be counted
   *
   *  \returns number of set bits in the integer i
   */
  inline unsigned int bitCount ( unsigned int i )
  {
    unsigned int bitCount = 0;
    for( ; i != 0; i /= 2 )
      bitCount += (i & 1);
    return bitCount;
  }



  /** \brief obtain index with greater value in array
   *
   *  \param[in]  array  array of comparable values
   *  \param[in]  i      first index
   *  \param[in]  j      second index
   *
   *  \note The array values must implement the operator <.
   *
   *  \returns i if array[ i ] > array[ j ], j otherwise
   */
  template< class Array, class Index >
  inline Index argmax( const Array &array, Index i, Index j )
  {
    return (array[ j ] < array[ i ] ? i : j);
  }

  /** \brief obtain index with lesser value in array
   *
   *  \param[in]  array  array of comparable values
   *  \param[in]  i      first index
   *  \param[in]  j      second index
   *
   *  \note The array values must implement the operator <.
   *
   *  \returns i if array[ i ] < array[ j ], j otherwise
   */
  template< class Array, class Index >
  inline Index argmin( const Array &array, Index i, Index j )
  {
    return (array[ i ] < array[ j ] ? i : j);
  }

} // namespace Dune

#endif // #ifndef DUNE_SPGRID_MISC_HH
